# How we host Ars, the finale and the 64-bit future
## We wrap up our four-part series by tying up loose ends and looking ahead.
## Information:

+ Link: https://arstechnica.com/information-technology/2023/08/ars-on-aws-04/
+ Source: Ars Technica
+ Author: Lee Hutchinson
+ Date published: 09 August 2023, 13:00
+ Date scraped: 10 August 2023, 12:02

## Article:
![Main article image](https://cdn.arstechnica.net/wp-content/uploads/2023/08/moonshark-over-new-york-760x380.jpg)







Greetings, dear readers, and congratulations—we've reached the end of our four-part series on how Ars Technica is hosted in the cloud, and it has been a journey. We've gone through our [infrastructure](https://arstechnica.com/information-technology/2023/07/ars-on-aws-01/), our [application stack](https://arstechnica.com/information-technology/2023/07/ars-on-aws-02/), and our [CI/CD strategy](https://arstechnica.com/information-technology/2023/08/ars-on-aws-03/) (that's "continuous integration and continuous deployment"—the process by which we manage and maintain our site's code).


Now, to wrap things up, we have a bit of a grab bag of topics to go through. In this final part, we'll discuss some leftover configuration details I didn't get a chance to dive into in earlier parts—including how our battle-tested liveblogging system works (it's surprisingly simple, and yet it has withstood millions of readers hammering at it during Apple events). We'll also peek at how we handle authoritative DNS.


Finally, we'll close on something that I've been wanting to look at for a while: AWS's cloud-based 64-bit ARM service offerings. How much of our infrastructure could we shift over onto ARM64-based systems, how much work will that be, and what might the long-term benefits be, both in terms of performance and costs?


But first, because I know we have readers who like to skip ahead, let's re-introduce our block diagram and make sure we're all caught up with what we're doing today:


[![This is starting to look familiar.](https://cdn.arstechnica.net/wp-content/uploads/2023/08/ars-aws-vpc-and-ci-cd-yellow.png)](https://cdn.arstechnica.net/wp-content/uploads/2023/08/ars-aws-vpc-and-ci-cd-yellow.png) / This is starting to look familiar.


## The recap: What we’ve got


So, recapping: Ars runs on WordPress for the front page, a smaller WordPress/WooCommerce instance for the [merch store](https://arstechnica.com/store/), and XenForo for the [OpenForum](https://arstechnica.com/civis/). All of these applications live as containers in ECS tasks (where "task" in this case is functionally equivalent to a Docker host, containing a number of services). Those tasks are invoked and killed as needed to scale the site up and down in response to the current amount of visitor traffic. Various other components of the stack contribute to keeping the site operational (like Aurora, which provides MySQL databases to the site, or Lambda, which we use to kick off WordPress scheduled tasks, among other things).


On the left side of the diagram, we have our CI/CD stack. The code that makes Ars work—which for us includes things like our WordPress PHP files, both core and plugin—lives in a private Github repo under version control. When we need to change our code (like if there's a WordPress update), we modify the source in the Github repo and then, using a whole set of tools, we push those changes out into the production web environment, and new tasks are spun up containing those changes. (As with the software stack, it's a little more complicated than that—[consult part three](https://arstechnica.com/information-technology/2023/08/ars-on-aws-03/) for a more granular description of the process!)


Eagle-eyed readers might notice there's something different about the diagram above: A few of the services are highlighted in yellow. Well-spotted—those are services that we might be able to switch over to run on ARM64 architecture, and we'll examine that near the end of this article.


Colors aside, there are also several things missing from that diagram. In attempting to keep it as high-level as possible while still being useful, I omitted a whole mess of more basic infrastructure components—and one of those components is DNS. It's one of those things we can't operate without, so let's jump in there and talk about it.







## Respect mah authoritative DNS


I'm going to talk a bit about DNS here, but I don't want to give the mistaken impression that I actually *understand* DNS. *No one* understands DNS. It is a mysterious eldritch service, powered by fear and blood, terrible in its rage and unstoppable in its wrath.


[![When something's acting weird, definitely check DNS. It's probably DNS. (If it's not DNS, then it's definitely the cache layer.)](https://cdn.arstechnica.net/wp-content/uploads/2023/08/dns-300x386.webp)](https://cdn.arstechnica.net/wp-content/uploads/2023/08/dns.webp) / When something's acting weird, definitely check DNS. It's probably DNS. (If it's not DNS, then it's definitely the cache layer.)[nixCraft](https://www.cyberciti.biz/humour/a-haiku-about-dns/)


Just about everyone has heard of DNS—that's the "[Domain Name System](https://en.wikipedia.org/wiki/Domain_Name_System)," the RFP-driven confederation of standards and protocols that functions like a phone book for looking up addresses on the Internet. (If you're the typical Ars reader, I'm going to bet you know a little more about DNS than the average bear, but just in case, please *bear* with me for the next paragraph of explanation while I make sure everyone is caught up. Ha ha! Bears!)


Most folks' daily interactions with DNS are on the client side—they send forward or reverse lookup requests to DNS servers, and the DNS servers answer those requests. (Briefly, a "forward" lookup is a request to take a hostname like `www.example.com` and provide the IP address, while a "reverse" lookup is the opposite, taking an IP address and giving you a hostname.) However, we're going to focus on the server side of DNS, which is something a lot fewer people end up ever having to mess with. Visitors need to be able to find your site by its name, so if you're running a site, you need a place to create and store your site's DNS records—that is, the canonical "live" list of all the hostnames and addresses and other records needed for a site to run. Your hosting provider likely offers this service, though it is also common to move one's DNS records off of one's webhost and onto an independent third-party DNS provider that offers different features.


Wherever your site's DNS records end up making their home is referred to as the "authoritative" nameserver (or servers) for your domain. And this brings us to our point: For Ars, we host our authoritative DNS in AWS with the rest of our infrastructure, using [Route 53](https://aws.amazon.com/route53/).


## Get your kicks on Route, uh, 53


Route 53 (named after the TCP port on which DNS services are typically offered) is a major DNS provider with a huge number of users, and it has the key advantage of being tied in with other AWS services—which means that as we do things that create or remove hostnames in other services, the DNS entries for those hostnames can be automatically created, updated, and removed via internal APIs. This is super handy in a complex environment with lots of moving parts—like ours. Here's a view of our DNS hosting, with test/dev/staging stuff redacted:


[![My zones, let me show you them.](https://cdn.arstechnica.net/wp-content/uploads/2023/08/hosted-zones.jpg)](https://cdn.arstechnica.net/wp-content/uploads/2023/08/hosted-zones.jpg) / My zones, let me show you them.


The screenshot refers to "hosted zones" instead of "domains" because ["zone" is a DNS-specific term](https://en.wikipedia.org/wiki/DNS_zone) denoting a particular controlled chunk of namespace. Indeed, when you get right down to it, all of your domain's forward and reverse DNS entries are typically stored in files, called "zone files." Every bit of canonical DNS information about your domain's hosts will be in there.


Zone files are typically kept as secure as is conveniently possible because while individual DNS entries are meant to be seen by the public (no one could visit your site without looking up your site's address!), the total contents of a zone file might give an attacker a great deal of information about your infrastructure's layout. Indeed, although it's obviously possible to query for individual records, a properly configured DNS server will outright reject a request for a copy of the entire zone file.


Here are some bits from our main Ars Technica zone file, for example:


[![Lot of redactions necessary here because zone files can contain sensitive information about how your site is put together.](https://cdn.arstechnica.net/wp-content/uploads/2023/08/ars-zones.jpg)](https://cdn.arstechnica.net/wp-content/uploads/2023/08/ars-zones.jpg) / Lot of redactions necessary here because zone files can contain sensitive information about how your site is put together.


We can see an [A-record](https://www.cloudflare.com/learning/dns/dns-records/dns-a-record/) for the top-level domain name `arstechnica.com`, which resolves to an entry point for our Application Load Balancer. (This means that visitors looking up `arstechnica.com` are given a DNS response that sends them to the ALB, which starts the process [described in part one](https://arstechnica.com/information-technology/2023/07/ars-on-aws-01/) wherein the ALB distributes visitors' traffic across the currently running ECS tasks.) Below that, we have an [MX record](https://www.cloudflare.com/learning/dns/dns-records/dns-mx-record/) for telling visitors what email servers to use when sending email to `arstechnica.com` inboxes.





The SOA ("start of authority") and NS ("nameserver") records show exactly which of Route 53's myriad DNS server instances are the ones that specifically own our records and are the authoritative nameservers for the domain, some [CNAMEs](https://www.cloudflare.com/learning/dns/dns-records/dns-cname-record/) to alias one hostname to another hostname, some [text records](https://www.cloudflare.com/learning/dns/dns-records/dns-txt-record/) to support some email security features like [DMARC](https://en.wikipedia.org/wiki/DMARC) and [DKIM](https://en.wikipedia.org/wiki/DomainKeys_Identified_Mail), and the list goes on and on. Lots of services that you wouldn't think use DNS actually *do* use DNS to store records, and we have dozens more DNS entries to support Ars' daily operations.


There are more zones in our Route 53 configuration, too—eight in total, including the main `arstechnica.com` zone. The others are primarily non-public-facing; `arstechnica.net`, for example, is what we use so our CDN can have its own domain for serving cached assets. The others are secret, and if I speak of them, Ars Tech Director Jason Marlin will sneak into my house in the middle of the night and rearrange all of my furniture. (The other zones are all for staging and for screwing around with test/dev work—you're not missing anything exciting.)


## Come on baby, WAF my fire(wall)


Another item that came up in the comments in previous pieces was whether or not Ars is using a "WAF"—that's a "web application firewall," for folks lucky enough to never have had to know or deal with such a thing.


The overriding difference between a "web application firewall" and what you might think of as a "normal" firewall is where in the [OSI model](https://en.wikipedia.org/wiki/OSI_model) the firewall is able to stick its fingers. While a standard [stateful firewall](https://en.wikipedia.org/wiki/Stateful_firewall) concerns itself with TCP, UDP, and ICMP packets at layers 3 and 4, a WAF operates up at layer 7 and can mess around with *application traffic.* From a practical sense, this means that a WAF can be configured to see and block layer 7 things like SQL injection attacks or brute-force login attempts—things for which a layer 3/4 stateful firewall is too blunt an instrument.


Like a stateful firewall, WAFs typically run based on a set of customizable rules that are executed in order on incoming traffic—the difference is that instead of rules like "block all traffic inbound from 10.20.30.0/24 with a destination of TCP port 53," WAF rules will be more application-focused, like, "block all inbound traffic containing malformed DNS requests." We're using AWS's built-in WAF to guard our infrastructure, so here's a screenshot of our WAF ruleset:


[![WAF rules tell the web application firewall what to allow and what to block.](https://cdn.arstechnica.net/wp-content/uploads/2023/08/waf-aws.jpg)](https://cdn.arstechnica.net/wp-content/uploads/2023/08/waf-aws.jpg) / WAF rules tell the web application firewall what to allow and what to block.


I've redacted a few entries that reveal internal information, but we can dig in on a couple of the lines to see what's in them. Let's peek at `AWS-AWSManagedRulesWordPressRuleSet`, which is an AWS-provided (as opposed to something we created) bundle of rules that AWS recommends for WordPress installations. Opening up that ruleset shows its details and contents:


[![This specific ruleset contains two rules, listed at bottom.](https://cdn.arstechnica.net/wp-content/uploads/2023/08/wafrule.jpg)](https://cdn.arstechnica.net/wp-content/uploads/2023/08/wafrule.jpg) / This specific ruleset contains two rules, listed at bottom.


The actual rules—two of them—are at the bottom, in the "Rules in AWSManagedRulesWordPressRuleSet" section. The console doesn't really explain what those two rules do, but [AWS documentation comes to the rescue](https://docs.aws.amazon.com/waf/latest/developerguide/aws-managed-rule-groups-use-case.html):


[![What the rules actually do.](https://cdn.arstechnica.net/wp-content/uploads/2023/08/waf-rule-docs.jpg)](https://cdn.arstechnica.net/wp-content/uploads/2023/08/waf-rule-docs.jpg) / What the rules actually do.[AWS](https://docs.aws.amazon.com/waf/latest/developerguide/aws-managed-rule-groups-use-case.html)


So our specific ruleset here is helping defend against "high-risk WordPress commands" that can be exploited in attacks, and it also looks at exploitable endpoints, like WordPress' [much-maligned `xmlrpc.php` file](https://kinsta.com/blog/xmlrpc-php/). And it's one of several rulesets that help block malicious traffic.


Before we leave the topic, it's important to acknowledge that if you're sensing there's significant functionality overlap between the WAF and the similarly layer-7-aware Application Load Balancer we discussed [back in part one](https://arstechnica.com/information-technology/2023/07/ars-on-aws-01/), you are definitely onto something. The two tools are notionally separate, but in practice, they interoperate closely—and under the covers, WAF and ALB are [aware of and deeply integrated with](https://aws.amazon.com/waf/faqs/) each other, as the "associated AWS resources" tab shows:


[![The WAF is the ALB. The ALB is the WAF. The worm <em>is</em> the spice. The spice <em>is</em> the worm.](https://cdn.arstechnica.net/wp-content/uploads/2023/08/alb-associated.jpg)](https://cdn.arstechnica.net/wp-content/uploads/2023/08/alb-associated.jpg) / The WAF is the ALB. The ALB is the WAF. The worm *is* the spice. The spice *is* the worm.












## Tags:
***ipv4-adresses***: 10.20.30.0

### Autogenerated tags:
dns aws wordpress arstechnica.com ruleset hostname hostnames tcp
